using System.Security.Cryptography.X509Certificates;
using System;
using System.Text;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Utilities.Executors;
using WingServerCommon.Log;
using System.Net.Http.Headers;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System.Collections.Concurrent;
using System.Dynamic;
using System.Text.RegularExpressions;
using System.Runtime.Serialization.Json;
using System.Net;
using WingServerCommon.Utilities;

namespace ReportService.Posters
{
    /// <summary>
    /// TODO do we need to post 
    /// </summary>

    public class ReportPoster : CacheReportPostersDTO, ICacheObject
    {
        private readonly string _defaultValueKey = "DefaultValue:";
        private readonly HttpClient _httpClient = new HttpClient();

        /// <summary>
        /// The report poster unique id
        /// </summary>
        /// <value></value>
        public string ReportPosterCode { get; set; }

        private IReadOnlyList<TemplateItem> TemplateItems { get; set; } = new List<TemplateItem>();
        public string AIReportLanguage
        {
            get
            {
                var aiReportLanguage = "zh-CN";
                var languageItem = TemplateItems.FirstOrDefault(x => x.TemplateField.ToLower() == "aireportlanguage");
                if (languageItem != null)
                {
                    aiReportLanguage = languageItem.VinnoReportField.Replace(_defaultValueKey, "");
                }
                return aiReportLanguage;
            }
        }

        public ReportPoster(ReportPosterDTO poster)
        {
            Code = poster.ReportPosterCode;
            ReportPosterCode = poster.ReportPosterCode;
            ReceiverName = poster.ReceiverName;
            TargetUrl = poster.TargetUrl;
            Contact = poster.Contact;
            Phone = poster.Phone;
            PosterType = (int)poster.PosterType;
            ReportFormat = (int)poster.ReportFormat;
            PostTimeout = poster.PostTimeout;
            Template = poster.Template;
            DeviceIds = poster.DeviceIds;
            _httpClient.Timeout = TimeSpan.FromSeconds(poster.PostTimeout);
            TemplateItems = LoadConfigFile(poster.Template);
        }

        public Tuple<bool, string, string> Post(ReportDTO reportInfo)
        {
            var reportId = reportInfo.ReportCode;
            string postContent = string.Empty;
            Tuple<bool, string> postResult = new Tuple<bool, string>(false, "");
            try
            {
                Logger.WriteLineInfo($"{ReceiverName} Begin to post report {reportId} with patientId {reportInfo.PatientId}({CryptionHelper.Decrypt(reportInfo.PatientName) }) to {TargetUrl}.");
                if (ReportFormat == (int)ReportFormatEnum.Json)
                {
                    postContent = GetJsonContent(reportInfo);
                    postResult = DoPost(reportId, postContent, "application/json");
                }
                else if (ReportFormat == (int)ReportFormatEnum.Xml)
                {
                    postContent = GetXMLContent(reportInfo);
                    postResult = DoPost(reportId, postContent, "application/xml");
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{ReceiverName} post report err, reportId: {reportId}, {ex}");
                postResult = new Tuple<bool, string>(false, ex.Message);
            }

            return new Tuple<bool, string, string>(postResult.Item1, postContent, postResult.Item2);
        }

        private Tuple<bool, string> DoPost(string reportId, string postContent, string mediaType)
        {
            try
            {
                HttpContent content = new ByteArrayContent(Encoding.UTF8.GetBytes(postContent));
                content.Headers.ContentType = new MediaTypeHeaderValue(mediaType);

                var postResult = _httpClient.PostAsync(TargetUrl, content);
                postResult.Wait();
                var response = postResult.Result;
                response.EnsureSuccessStatusCode();
                var resultContent = response.Content.ReadAsStringAsync();
                resultContent.Wait();
                var result = resultContent.Result;

                var args = new PostReportResultArgs();
                if (result == "\"success\"" || result == "success") //TODO remove this if we dont support old customer in future
                {
                    return new Tuple<bool, string>(true, result);
                }
                else
                {
                    using (var ms = new MemoryStream(Encoding.UTF8.GetBytes(result)))
                    {
                        try
                        {
                            var posterResult = (PosterResult)new DataContractJsonSerializer(typeof(PosterResult)).ReadObject(ms);
                            if (posterResult.success || posterResult.code == "0")
                            {
                                return new Tuple<bool, string>(true, result);
                            }
                        }
                        catch (Exception exception)
                        {
                            Logger.WriteLineWarn($"{ReceiverName} Convert result about poster report: {reportId} failed, PostResult:{resultContent.Result}, Exception:{exception}");
                        }
                    }
                    return new Tuple<bool, string>(false, result);
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"{ReceiverName} Post report {reportId} ex:{ex}");
                return new Tuple<bool, string>(false, ex.Message);
            }
        }

        private string GetJsonContent(ReportDTO reportInfo)
        {
            var postDatas = GetPostDatas(reportInfo);
            IDictionary<string, object> expandoObjects = new ExpandoObject();
            foreach (var d in postDatas)
            {
                if (d.Value is List<string> images)
                {
                    var base64Images = new List<string>();
                    foreach (var img in images)
                    {
                        base64Images.Add(ToBase64Images(img));
                    }

                    expandoObjects.Add(d.Key, base64Images);
                }
                else
                {

                    expandoObjects.Add(d);
                }
            }
            return JsonConvert.SerializeObject(expandoObjects as ExpandoObject);
        }

        protected virtual string GetXMLContent(ReportDTO reportInfo)
        {
            var postDatas = GetPostDatas(reportInfo);
            var xmlReportBuilder = new StringBuilder();
            xmlReportBuilder.Append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            xmlReportBuilder.Append("<ReportInfo>");
            foreach (var key in postDatas.Keys)
            {
                var ele = postDatas[key];
                if (ele is String value)
                {
                    Regex reg = new Regex("[<>&]");
                    if (reg.Match(value).Success)
                    {
                        value = FormatStringValue(value);
                    }
                    xmlReportBuilder.AppendFormat("<{0}>{1}</{0}>", key, value);
                }
                else if (ele is List<string> images && images?.Any() == true)
                {
                    xmlReportBuilder.Append("<Images>");
                    foreach (var imgUrl in images)
                    {
                        var imageVal = $"<Image Path=\"{imgUrl}\" Type =\"4\" >{ToBase64Images(imgUrl)}</Image>";
                        xmlReportBuilder.Append(imageVal);
                    }
                    xmlReportBuilder.Append("</Images>");
                }
            }
            xmlReportBuilder.Append("</ReportInfo>");
            return xmlReportBuilder.ToString();
        }

        protected ConcurrentDictionary<string, object> GetPostDatas(ReportDTO reportInfo)
        {
            var postDatas = new ConcurrentDictionary<string, object>();
            var reportDictionary = ReportInfoToDictionary(reportInfo);
            foreach (var item in TemplateItems)
            {
                if (item.VinnoReportField.Contains(_defaultValueKey))
                {
                    var realValue = item.VinnoReportField.Replace(_defaultValueKey, "");
                    postDatas.AddOrUpdate(item.TemplateField, realValue, (k, v) => { return realValue; });
                }
                else if (reportDictionary.Keys.Contains(item.VinnoReportField))
                {
                    var ele = reportDictionary[item.VinnoReportField];
                    postDatas.AddOrUpdate(item.TemplateField, ele, (k, v) => { return ele; });
                }
            }
            return postDatas;
        }

        private ConcurrentDictionary<string, object> ReportInfoToDictionary(ReportDTO reportInfo)
        {
            var reportDatas = new ConcurrentDictionary<string, object>();
            if (!string.IsNullOrWhiteSpace(reportInfo.ReportDatasJson))
            {
                var reportElements = Newtonsoft.Json.JsonConvert.DeserializeObject<List<ReportElementDTO>>(reportInfo.ReportDatasJson, new ReportElementDTOConverter());
                foreach (var r in reportElements)
                {
                    if (r is ReportElementStringDTO es)
                    {
                        reportDatas.AddOrUpdate(es.Name, es.Value, (k, v) => { return es.Value; });
                    }
                    if (r is ReportElementArrayDTO ea)
                    {
                        reportDatas.AddOrUpdate(ea.Name, ea.Value, (k, v) => { return ea.Value; });
                    }
                }
            }
            if (!reportDatas.Keys.Contains("RecordCode"))
            {
                reportDatas.AddOrUpdate("RecordCode", reportInfo.RecordCode, (k, v) => { return reportInfo.RecordCode; });
            }
            if (!reportDatas.Keys.Contains("PatientId"))
            {
                reportDatas.AddOrUpdate("PatientId", reportInfo.PatientId, (k, v) => { return reportInfo.PatientId; });
            }
            if (!reportDatas.Keys.Contains("PatientName"))
            {
                reportDatas.AddOrUpdate("PatientName", CryptionHelper.Decrypt(reportInfo.PatientName) , (k, v) => { return CryptionHelper.Decrypt(reportInfo.PatientName); });
            }
            if (!reportDatas.Keys.Contains("RecordCode"))
            {
                reportDatas.AddOrUpdate("RecordCode", reportInfo.RecordCode, (k, v) => { return reportInfo.RecordCode; });
            }
            if (!reportDatas.Keys.Contains("DeviceCode"))
            {
                reportDatas.AddOrUpdate("DeviceCode", reportInfo.DeviceCode, (k, v) => { return reportInfo.DeviceCode; });
            }
            if (!reportDatas.Keys.Contains("CreateTime"))
            {
                reportDatas.AddOrUpdate("CreateTime", reportInfo.CreateTime.ToString("yyyy-MM-dd HH:mm:ss"), (k, v) => { return reportInfo.CreateTime.ToString("yyyy-MM-dd HH:mm:ss"); });
            }
            if (!reportDatas.Keys.Contains("UpdateTime"))
            {
                reportDatas.AddOrUpdate("UpdateTime", reportInfo.UpdateTime.ToString("yyyy-MM-dd HH:mm:ss"), (k, v) => { return reportInfo.UpdateTime.ToString("yyyy-MM-dd HH:mm:ss"); });
            }
            return reportDatas;
        }

        /// <summary>
        /// load config xml file
        /// </summary>
        /// <param name="configContent">config Content</param>        
        /// <returns></returns>
        private static IReadOnlyList<TemplateItem> LoadConfigFile(string configContent)
        {
            var list = new List<TemplateItem>();
            if (string.IsNullOrEmpty(configContent))
            {
                return list;
            }

            var configLoader = new ConfigLoader();
            var sections = configLoader.LoadConfig(configContent);
            if (sections.Count == 1)
            {
                foreach (var item in sections[0].Items)
                {
                    list.Add(new TemplateItem(item.Key, item.Value));
                }
            }

            return list;
        }

        protected string FormatStringValue(string content)
        {
            if (!string.IsNullOrEmpty(content))
            {
                return $"<![CDATA[{content}]]>";
            }
            return string.Empty;
        }

        protected string ToBase64Images(string imageUrl)
        {
            var bytes = DownloadAsync(imageUrl).Result;
            string result = Convert.ToBase64String(bytes);
            return result;
        }

        /// <summary>下载文件</summary>
        /// <param name="fileUrl"></param>
        /// <returns></returns>
        private async Task<byte[]> DownloadAsync(string fileUrl)
        {
            try
            {
                if (string.IsNullOrEmpty(fileUrl))
                {
                    return new byte[0];
                }
                long fileSize = 0;
                using (var request = new HttpRequestMessage())
                {
                    request.RequestUri = new Uri(fileUrl);
                    request.Method = HttpMethod.Get;
                    var response = await _httpClient.SendAsync(request);

                    if (response != null && response.StatusCode == HttpStatusCode.OK)
                    {
                        var contentLength = response.Content.Headers.ContentLength;
                        fileSize = contentLength == null ? 0 : contentLength.Value;
                    }
                }
                if (fileSize <= 0)
                {
                    throw new NotSupportedException($"fileSize is {fileSize}");
                }
                byte[] bytes = await _httpClient.GetByteArrayAsync(fileUrl);
                return bytes;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"ReportPoster download file err, url: {fileUrl}, {ex}");
            }
            finally
            {
                //Logger.WriteLineInfo($"download file:{fileUrl}");
            }
            return new byte[0];
        }
    }

    public class PostReportResultArgs : EventArgs
    {

    }
}