using System.Collections.Generic;
using WingServerCommon.Utilities.Executors;
using WingServerCommon.Log;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.DTO.Report;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.DB.Request;

namespace ReportService.Posters
{
    public class ReportPostersObserver
    {
        private Func<string, string, ReportDTO> OnFindReport;
        private Func<UpdateReportPushRecordDBRequest, bool> OnUpdateReportPushRecord;

        public ReportPostersObserver(Func<string, string, ReportDTO> findReport, Func<UpdateReportPushRecordDBRequest, bool> updateReportPushRecord)
        {
            OnFindReport = findReport;
            OnUpdateReportPushRecord = updateReportPushRecord;

        }

        public IList<CacheReportPostersDTO> GetDevicePosters(string deviceCode)
        {
            var deviceInfo = CacheMaintenance.Instance.Get<IDeviceInfosManager>().Get(deviceCode);
            if (deviceInfo?.ReportPosterCodes.Any() == true)
            {
                var posters = CacheMaintenance.Instance.Get<IReportPostersManager>().Where(x => deviceInfo.ReportPosterCodes.Contains(x.Code));
                return posters ?? new List<CacheReportPostersDTO>();
            }
            return new List<CacheReportPostersDTO>();
        }

        public void Post(ReportPushRecordDTO history)
        {
            var poster = CacheMaintenance.Instance.Get<IReportPostersManager>().Where(x => x.Code == history.PoseterConfigId)?.FirstOrDefault();
            if (poster != null)
            {
                Tuple<bool, string, string> result = new Tuple<bool, string, string>(false, "", "");
                if (poster is ReportPoster rp)
                {
                    var reportInfo = OnFindReport?.Invoke(history.ReportId, rp.AIReportLanguage);
                    if (reportInfo != null && !string.IsNullOrWhiteSpace(reportInfo.ReportCode))
                    {
                        result = rp.Post(reportInfo);
                    }
                }
                if (poster is BJAidReportPoster bjrp)
                {
                    var reportInfo = OnFindReport?.Invoke(history.ReportId, bjrp.AIReportLanguage);
                    if (reportInfo != null && !string.IsNullOrWhiteSpace(reportInfo.ReportCode))
                    {
                        result = bjrp.Post(reportInfo);
                    }
                }
                OnUpdateReportPushRecord?.Invoke(new UpdateReportPushRecordDBRequest
                {
                    PushRecordCode = history.PushRecordCode,
                    State = result.Item1 ? PushDataStateEnum.Successful : PushDataStateEnum.Failed,
                    PushResultToken = result.Item2,
                    PushResponse = result.Item3,
                });
                return;
            }
            OnUpdateReportPushRecord?.Invoke(new UpdateReportPushRecordDBRequest
            {
                PushRecordCode = history.PushRecordCode,
                State = PushDataStateEnum.Failed,
                PushResultToken = "",
                PushResponse = "not matched poster",
            });
        }
    }

}